@@ -2,6 +2,7 @@ use netscan::blocking::PortScanner;
 use netscan::setting::{ScanType, Destination};
 use std::time::Duration;
 use std::net::{IpAddr, Ipv4Addr};
+use std::thread;
 
 fn main() {
     let mut port_scanner = match PortScanner::new(IpAddr::V4(Ipv4Addr::new(192, 168, 1, 4))) {
@@ -21,9 +22,18 @@ fn main() {
     port_scanner.set_scan_type(ScanType::TcpSynScan);
     port_scanner.set_timeout(Duration::from_millis(10000));
     port_scanner.set_wait_time(Duration::from_millis(100));
-    //port_scanner.set_send_rate(Duration::from_millis(1)); 
+    //port_scanner.set_send_rate(Duration::from_millis(1));
+    
+    let rx = port_scanner.get_progress_receiver();
     // Run scan 
-    let result = port_scanner.scan();
+    let handle = thread::spawn(move|| {
+        port_scanner.scan()
+    });
+    // Print progress
+    while let Ok(_socket_addr) = rx.lock().unwrap().recv() {
+        //println!("Check: {}", socket_addr);
+    }
+    let result = handle.join().unwrap();
     // Print results 
     println!("Status: {:?}", result.scan_status);
     println!("Results:");
