@@ -1,5 +1,6 @@
 use std::net::{IpAddr, SocketAddr};
 use std::sync::{Arc, Mutex};
+use std::sync::mpsc::Sender;
 use std::thread;
 use std::time::{Instant, Duration};
 use socket2::{Domain, Protocol, SockAddr, Socket, Type};
@@ -32,7 +33,7 @@ fn build_udp_packet(src_ip: IpAddr, src_port: u16, dst_ip: IpAddr, dst_port: u16
     udp_packet.packet().to_vec()
 }
 
-fn send_icmp_echo_packets(socket: &Socket, scan_setting: &ScanSetting) {
+fn send_icmp_echo_packets(socket: &Socket, scan_setting: &ScanSetting, ptx: &Arc<Mutex<Sender<SocketAddr>>>) {
     for dst in scan_setting.destinations.clone() {
         let socket_addr = SocketAddr::new(dst.dst_ip, 0);
         let sock_addr = SockAddr::from(socket_addr);
@@ -41,11 +42,20 @@ fn send_icmp_echo_packets(socket: &Socket, scan_setting: &ScanSetting) {
             Ok(_) => {},
             Err(_) => {},
         }
+        match ptx.lock() {
+            Ok(lr) => {
+                match lr.send(socket_addr) {
+                    Ok(_) => {},
+                    Err(_) => {},
+                }
+            },
+            Err(_) => {},
+        }
         thread::sleep(scan_setting.send_rate);
     }
 }
 
-fn send_tcp_syn_packets(socket: &Socket, scan_setting: &ScanSetting){
+fn send_tcp_syn_packets(socket: &Socket, scan_setting: &ScanSetting, ptx: &Arc<Mutex<Sender<SocketAddr>>>){
     for dst in scan_setting.destinations.clone() {
         for port in dst.dst_ports {
             let socket_addr = SocketAddr::new(dst.dst_ip, port);
@@ -55,12 +65,21 @@ fn send_tcp_syn_packets(socket: &Socket, scan_setting: &ScanSetting){
                 Ok(_) => {},
                 Err(_) => {},
             }
+            match ptx.lock() {
+                Ok(lr) => {
+                    match lr.send(socket_addr) {
+                        Ok(_) => {},
+                        Err(_) => {},
+                    }
+                },
+                Err(_) => {},
+            }
             thread::sleep(scan_setting.send_rate);
         }
     }
 }
 
-fn send_udp_packets(socket: &Socket, scan_setting: &ScanSetting) {
+fn send_udp_packets(socket: &Socket, scan_setting: &ScanSetting, ptx: &Arc<Mutex<Sender<SocketAddr>>>) {
     for dst in scan_setting.destinations.clone() {
         for port in dst.dst_ports {
             let socket_addr = SocketAddr::new(dst.dst_ip, port);
@@ -70,6 +89,15 @@ fn send_udp_packets(socket: &Socket, scan_setting: &ScanSetting) {
                 Ok(_) => {},
                 Err(_) => {},
             }
+            match ptx.lock() {
+                Ok(lr) => {
+                    match lr.send(socket_addr) {
+                        Ok(_) => {},
+                        Err(_) => {},
+                    }
+                },
+                Err(_) => {},
+            }
             thread::sleep(scan_setting.send_rate);
         }
     }
@@ -112,16 +140,16 @@ fn run_connect_scan(scan_setting: ScanSetting, scan_result: &Arc<Mutex<ScanResul
     }
 }
 
-fn send_ping_packet(socket: &Socket, scan_setting: &ScanSetting) {
+fn send_ping_packet(socket: &Socket, scan_setting: &ScanSetting, ptx: &Arc<Mutex<Sender<SocketAddr>>>) {
     match scan_setting.scan_type {
         ScanType::IcmpPingScan => {
-            send_icmp_echo_packets(socket, scan_setting);
+            send_icmp_echo_packets(socket, scan_setting, ptx);
         },
         ScanType::TcpPingScan => {
-            send_tcp_syn_packets(socket, scan_setting);
+            send_tcp_syn_packets(socket, scan_setting, ptx);
         },
         ScanType::UdpPingScan => {
-            send_udp_packets(socket, scan_setting);
+            send_udp_packets(socket, scan_setting, ptx);
         },
         _ => {
             return;
@@ -129,10 +157,10 @@ fn send_ping_packet(socket: &Socket, scan_setting: &ScanSetting) {
     }
 }
 
-fn send_tcp_packets(socket: &Socket, scan_setting: &ScanSetting) {
+fn send_tcp_packets(socket: &Socket, scan_setting: &ScanSetting, ptx: &Arc<Mutex<Sender<SocketAddr>>>) {
     match scan_setting.scan_type {
         ScanType::TcpSynScan => {
-            send_tcp_syn_packets(socket, scan_setting);
+            send_tcp_syn_packets(socket, scan_setting, ptx);
         },
         _ => {
             return;
@@ -140,7 +168,7 @@ fn send_tcp_packets(socket: &Socket, scan_setting: &ScanSetting) {
     }
 }
 
-pub(crate) fn scan_hosts(scan_setting: ScanSetting) -> HostScanResult {
+pub(crate) fn scan_hosts(scan_setting: ScanSetting, ptx: &Arc<Mutex<Sender<SocketAddr>>>) -> HostScanResult {
     let socket = match scan_setting.src_ip {
         IpAddr::V4(_) => {
             match scan_setting.scan_type {
@@ -191,14 +219,14 @@ pub(crate) fn scan_hosts(scan_setting: ScanSetting) -> HostScanResult {
     thread::spawn(move || {
         receiver::receive_packets(&mut rx, receive_setting, &receive_result, &receive_stop);    
     });
-    send_ping_packet(&socket, &scan_setting);
+    send_ping_packet(&socket, &scan_setting, ptx);
     thread::sleep(scan_setting.wait_time);
     *stop.lock().unwrap() = true;
     let result: HostScanResult = scan_result.lock().unwrap().host_scan_result.clone(); 
     return result;
 }
 
-pub(crate) fn scan_ports(scan_setting: ScanSetting) -> PortScanResult {
+pub(crate) fn scan_ports(scan_setting: ScanSetting, ptx: &Arc<Mutex<Sender<SocketAddr>>>) -> PortScanResult {
     let socket = match scan_setting.src_ip {
         IpAddr::V4(_) => {
             match scan_setting.scan_type {
@@ -249,7 +277,7 @@ pub(crate) fn scan_ports(scan_setting: ScanSetting) -> PortScanResult {
             thread::spawn(move || {
                 receiver::receive_packets(&mut rx, receive_setting, &receive_result, &receive_stop);    
             });
-            send_tcp_packets(&socket, &scan_setting);
+            send_tcp_packets(&socket, &scan_setting, ptx);
             thread::sleep(scan_setting.wait_time);
             *stop.lock().unwrap() = true;
         },
