@@ -1,5 +1,5 @@
-use pnet::packet::ethernet::{MutableEthernetPacket, EtherType, EtherTypes};
-use pnet::datalink::MacAddr;
+use pnet_packet::ethernet::{MutableEthernetPacket, EtherType, EtherTypes};
+use pnet_datalink::MacAddr;
 
 pub const ETHERNET_HEADER_LEN: usize = 14;
 
@@ -9,13 +9,13 @@ pub fn build_ethernet_packet(eth_packet: &mut MutableEthernetPacket, src_mac: Ma
     eth_packet.set_destination(dst_mac);
     match ether_type {
         EtherTypes::Arp => {
-            eth_packet.set_ethertype(pnet::packet::ethernet::EtherTypes::Arp);
+            eth_packet.set_ethertype(EtherTypes::Arp);
         },
         EtherTypes::Ipv4 => {
-            eth_packet.set_ethertype(pnet::packet::ethernet::EtherTypes::Ipv4);
+            eth_packet.set_ethertype(EtherTypes::Ipv4);
         },
         EtherTypes::Ipv6 => {
-            eth_packet.set_ethertype(pnet::packet::ethernet::EtherTypes::Ipv6);
+            eth_packet.set_ethertype(EtherTypes::Ipv6);
         },
         _ => {
             //ToDo
